#include "osHandles.h"              // Includes all OS Files
#include "System/cpu.h"             // Initialize CPU Hardware
#include "drivers/watchdog.h"
#include "drivers/uart0.h"       	// Initialize UART
#include "drivers/i2c.h"            // Initialize I2C
#include "drivers/pcm1774.h"        // Initialize PCM1774
#include "drivers/sta013.h"         // Initialize Sta013
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "../fat/diskio.h"
#include "../fat/ff.h"            // FAT File System Library
#include "general/userInterface.hpp"	// User interface functions to interact through UART
#include "MP3/mp3.hpp"
#include "utils/timer.hpp"
#include "../utils/vector.hpp"
#include "../FlashReading/FlashRead.hpp"
#include "../drivers/pwm.h"
#include "../drivers/ADC.h"
#include "../drivers/PING.h"
#include "sensors.hpp"
#include "../Testing/Testing.hpp"
#include "../MotorControl/MotorControl.hpp"
#include "../drivers/ssp_spi.h"
#include "Chip_Select.h"
#include "../drivers/Read_Write.h"

void FatFsDiskTimer(void* p)
{
	disk_timerproc();
}

void MovementTest(void* p);
void SensorUpdate(void* p);
int SongNumber = 0;
unsigned int triggerTime = 0;
unsigned int Data, ADC_DATA;


int main(void)
{
	OSHANDLES SysHandles; // Should contain all OS Handles

	cpuSetupHardware(); // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000 * 1000); // Some startup delay
	uart0Init(38400, 128); // 128 is size of Rx/Tx FIFO

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000 * 1000);

	/** Create timer needed for SD Card I/O */
	Timer diskTimer(FatFsDiskTimer, 10, TimerPeriodic);
	diskTimer.start();

	/** Create any Queues and Mutexes **/
	SysHandles.lock.SPI = xSemaphoreCreateMutex();
	// TODO 2.  Create the "song name" Queue here
	SysHandles.queue.song = xQueueCreate(1, 16);

	if (pdPASS!= xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.userInterface )
		||
		pdPASS!= xTaskCreate( mp3Task, (signed char*)"MP3", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.mp3 )
		||
		pdPASS!= xTaskCreate( MovementTest, (signed char*)"Testing back and Front Wheel together", STACK_BYTES(512), &SysHandles, PRIORITY_LOW, &SysHandles.task.MovementTest )
		||
		pdPASS!= xTaskCreate( SensorUpdate, (signed char*)"Update the Sensors", STACK_BYTES(512), &SysHandles, PRIORITY_LOW, &SysHandles.task.SensorUpdate ))
	{
		rprintf(
				"ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}

/******************************************************************************
 ** Function name:		MovementTest()
 **
 ** Descriptions:		Back and Front Wheel Testing Together.
 ******************************************************************************/
void MovementTest(void* p)
{
	OSHANDLES *osHandles = (OSHANDLES*) p;
	initialize_PWM_FrontBackWheel();
	unsigned int DangerLeft = 80;
	unsigned int DangerRight = 80;
	unsigned int DangerMiddle = 220;

	/************************Song Plays When Car Start first**********************************/
	vTaskDelay(800); //Minimum time to initiate drivers before following functions execute
	unsigned int volume = 100; // Max volume
	pcm1774_OutputVolume(volume, volume);
	char StartSong[20] = "start.mp3";
	xQueueSend(osHandles->queue.song,StartSong,1000);
	vTaskDelay(3600); // This delay is to complete the song then car will start moving
	/*****************************************************************************************/

	for (;;)
	{
		BackMotorForward(osHandles->sensorValues.middle);
		if (osHandles->sensorValues.left < DangerLeft)
		{
			FrontMotorRight();
		}
		else if (osHandles->sensorValues.right < DangerRight)
		{
			FrontMotorLeft();
		}
		else if (osHandles->sensorValues.middle < DangerMiddle)
		{
			if (osHandles->sensorValues.left > osHandles->sensorValues.right)
			{
				FrontMotorLeft();
			}
			else
			{
				FrontMotorRight();
			}
			// Steer in one direciton for a longer duration to avoid front facing object
			vTaskDelay(25);
		}
		vTaskDelay(25);
		FrontMotorCenter();
	}
}
